#!/usr/bin/env python3

"""
Dragonglass - a utility for processing Obsidian vaults into directories of Web pages.

Add a configuration file named .dragonglass to the root folder of the Obsidian vault, and a template subdirectory
named .dragonglass.tmpl.
"""

import argparse
import logging
import shutil
from pathlib import Path

from .config import Context
from .mparse import create_markdown_parser
from .style import write_default_stylesheet
from .template import create_template_environment, template_vars
from .tree import SourceIndex, generate_list

logger = logging.getLogger(__name__)

# The command line parser
parser = argparse.ArgumentParser(prog='dragonglass')
parser.add_argument('source_dir', help='Source directory (Obsidian vault) for the conversion.')
parser.add_argument('dest_dir', help='Destination directory for the conversion.')
parser.add_argument('-C', '--config', help='Specifies an alternate name for the configuration file.')
parser.add_argument('-D', '--debug', action='store_true', help='Enables debug logging')
parser.add_argument('--dump', action='store_true', help='Dumps the parsed data before writing the destination')


def main() -> int:
    """
    Main function of dragonglass: parses the Obsidian vault and outputs it as a formatted Web page directory.

    Returns:
        int: 0 on success, nonzero value on error.
    """
    try:
        args = parser.parse_args()

        if args.debug:
            logging.basicConfig(level=logging.DEBUG)
        else:
            logging.basicConfig(level=logging.INFO)

        context = Context()
        context.source_dir = Path(args.source_dir).resolve()
        if not (context.source_dir.exists() and context.source_dir.is_dir()):
            logger.error(f"{context.source_dir} is not a valid directory")
            return 1

        dest_dir = Path(args.dest_dir).resolve()
        if dest_dir.exists() and not dest_dir.is_dir():
            logger.error(f"{dest_dir} exists but is not a valid directory")
            return 1

        context.load_config(args)

        nodes = generate_list(context.source_dir)
        for node in nodes:
            logger.info(f"Loading metadata for {node}")
            context.current_node = node
            node.load_metadata()

        context.src_index = SourceIndex(nodes)

        mdparse = create_markdown_parser(context)
        for node in nodes:
            logger.info(f"Parsing {node}")
            context.current_node = node
            node.parse_markdown(mdparse)

        # TEMP
        if args.dump:
            for node in nodes:
                print(node)
                if node.metadata:
                    print(f"Metadata: {node.metadata}")
                if node.text:
                    print("----- BEGIN TEXT -----")
                    print(node.text)
                    print("------ END TEXT ------")

        if not dest_dir.exists():
            dest_dir.mkdir()
        tenv = create_template_environment(context)

        for node in nodes:
            if node.publish:
                p = node.target_file(dest_dir)
                if node.is_dir:
                    p.mkdir(exist_ok=True)
                elif node.is_md:
                    context.current_node = node
                    tmpl = tenv.get_template(context.get_template_name_for_node(node))
                    data = tmpl.render(template_vars(node, context))
                    with p.open("wt") as f:
                        f.write(data)
                else:
                    shutil.copyfile(context.source_dir / node.path, p)

        write_default_stylesheet(context, tenv, dest_dir)

    except:
        logger.exception("Unexpected error in processing")
        return 1

    return 0
